@@ -180,7 +180,7 @@ data EventInfo
                        }
   | SparkCounters      { sparksCreated, sparksDud, sparksOverflowed,
                          sparksConverted, sparksFizzled, sparksGCd,
-                         sparksRemaining :: {-# UNPACK #-}! Word64
+                         sparksRemaining :: {-# UNPACK #-} !Word64
                        }
   | SparkCreate        { }
   | SparkDud           { }
@@ -220,6 +220,11 @@ data EventInfo
                        , parTotCopied :: {-# UNPACK #-}!Word64
                        , parBalancedCopied :: !(Maybe Word64)
                        }
+  | MemReturn          { heapCapset :: !Capset
+                       , current :: !Word32
+                       , needed :: !Word32
+                       , returned :: !Word32
+                       }
 
   -- heap statistics
   | HeapAllocated      { heapCapset  :: {-# UNPACK #-}!Capset
@@ -228,6 +233,9 @@ data EventInfo
   | HeapSize           { heapCapset  :: {-# UNPACK #-}!Capset
                        , sizeBytes   :: {-# UNPACK #-}!Word64
                        }
+  | BlocksSize         { heapCapset  :: {-# UNPACK #-}!Capset
+                       , blocksSize  :: {-# UNPACK #-}!Word64
+                       }
   | HeapLive           { heapCapset  :: {-# UNPACK #-}!Capset
                        , liveBytes   :: {-# UNPACK #-}!Word64
                        }
@@ -327,7 +335,7 @@ data EventInfo
                               receiverInport :: {-# UNPACK #-} !PortId
                             }
 
-  -- These events have been added for Mercury's benifit but are generally
+  -- These events have been added for Mercury's benefit but are generally
   -- useful.
   | InternString       { str :: String, sId :: {-# UNPACK #-}!StringId }
 
@@ -396,6 +404,13 @@ data EventInfo
                        , heapProfSrcLoc :: !Text
                        , heapProfFlags :: !HeapProfFlags
                        }
+  | InfoTableProv      { itInfo :: !Word64
+                       , itTableName :: !Text
+                       , itClosureDesc :: !Int
+                       , itTyDesc :: !Text
+                       , itLabel :: !Text
+                       , itModule :: !Text
+                       , itSrcLoc :: !Text }
   | HeapProfSampleBegin
                        { heapProfSampleEra :: !Word64
                        }
@@ -431,6 +446,35 @@ data EventInfo
 
   | UserBinaryMessage  { payload :: !B.ByteString
                        }
+
+  | ConcMarkBegin
+  | ConcMarkEnd        { concMarkedObjectCount :: !Word32
+                       }
+  | ConcSyncBegin
+  | ConcSyncEnd
+  | ConcSweepBegin
+  | ConcSweepEnd
+  | ConcUpdRemSetFlush { cap    :: {-# UNPACK #-}!Int
+                       }
+  | NonmovingHeapCensus
+                       { nonmovingCensusBlkSize :: !Word8
+                       , nonmovingCensusActiveSegs :: !Word32
+                       , nonmovingCensusFilledSegs :: !Word32
+                       , nonmovingCensusLiveBlocks :: !Word32
+                       }
+  | TickyCounterDef
+                       { tickyCtrDefId      :: !Word64
+                       , tickyCtrDefArity   :: !Word16
+                       , tickyCtrDefKinds   :: !Text
+                       , tickyCtrDefName    :: !Text
+                       }
+  | TickyCounterSample
+                       { tickyCtrSampleId         :: !Word64
+                       , tickyCtrSampleEntryCount :: !Word64
+                       , tickyCtrSampleAllocs     :: !Word64
+                       , tickyCtrSampleAllocd     :: !Word64
+                       }
+  | TickyBeginSample
   deriving Show
 
 {- [Note: Stop status in GHC-7.8.2]
@@ -444,7 +488,7 @@ See GHC bug #9003 for a discussion.
 
 The parsers in Events.hs have to be adapted accordingly, providing
 special ghc-7.8.2 parsers for the thread-stop event if GHC-7.8.2
-produced the eventlog.
+produced the event log.
 The EVENT_USER_MARKER was not present in GHC-7.6.3, and a new event
 EVENT_HACK_BUG_T9003 was added in GHC-7.8.3, so we take presence of
 USER_MARKER and absence of HACK_BUG_T9003 as an indication that
@@ -498,7 +542,7 @@ mkStopStatus n = case n of
  16 ->  BlockedOnMsgThrowTo
  17 ->  ThreadMigrating
  18 ->  BlockedOnMsgGlobalise
- 19 ->  NoStatus -- yeuch... this one does not actually exist in GHC eventlogs
+ 19 ->  NoStatus -- yeuch... this one does not actually exist in GHC event logs
  20 ->  BlockedOnMVarRead -- since GHC-7.8.3
  _  ->  error "mkStat"
 
@@ -586,6 +630,7 @@ data HeapProfBreakdown
   | HeapProfBreakdownRetainer
   | HeapProfBreakdownBiography
   | HeapProfBreakdownClosureType
+  | HeapProfBreakdownInfoTable
   deriving Show
 
 instance Binary HeapProfBreakdown where
@@ -599,6 +644,7 @@ instance Binary HeapProfBreakdown where
       5 -> return HeapProfBreakdownRetainer
       6 -> return HeapProfBreakdownBiography
       7 -> return HeapProfBreakdownClosureType
+      8 -> return HeapProfBreakdownInfoTable
       _ -> fail $ "Unknown HeapProfBreakdown: " ++ show n
   put breakdown = put $ case breakdown of
     HeapProfBreakdownCostCentre -> (1 :: Word32)
@@ -608,6 +654,7 @@ instance Binary HeapProfBreakdown where
     HeapProfBreakdownRetainer -> 5
     HeapProfBreakdownBiography -> 6
     HeapProfBreakdownClosureType -> 7
+    HeapProfBreakdownInfoTable -> 8
 
 newtype HeapProfFlags = HeapProfFlags Word8
   deriving (Show, Binary)
