@@ -105,15 +105,15 @@ import Data.Monoid (mempty)
 import Data.Monoid ((<>))
 #endif
 
--- | Read an entire eventlog file. It returns an error message if it
--- encouters an error while decoding.
+-- | Read an entire event log file. It returns an error message if it
+-- encounters an error while decoding.
 --
 -- Note that it doesn't fail if it consumes all input in the middle of decoding
 -- of an event.
 readEventLogFromFile :: FilePath -> IO (Either String EventLog)
 readEventLogFromFile path = fmap fst . readEventLog <$> BL.readFile path
 
--- | Read an eventlog file and pretty print it to stdout
+-- | Read an event log file and pretty print it to stdout
 printEventsIncremental
   :: Bool -- ^ Follow the file or not
   -> FilePath
@@ -121,7 +121,7 @@ printEventsIncremental
 printEventsIncremental follow path =
   withFile path ReadMode (hPrintEventsIncremental follow)
 
--- | Read an eventlog from the Handle and pretty print it to stdout
+-- | Read an event log from the Handle and pretty print it to stdout
 hPrintEventsIncremental
   :: Bool -- ^ Follow the handle or not
   -> Handle
@@ -178,7 +178,7 @@ capSplitEvents' evts imap =
   (x:xs) -> capSplitEvents' xs (IM.insertWith (++) (getIntCap x) [x] imap)
   []     -> imap
 
--- Adds a block marker to the beginnng of a list of events, annotated with
+-- Adds a block marker to the beginning of a list of events, annotated with
 -- its capability. All events are expected to belong to the same cap.
 addBlockMarker :: Int -> [Event] -> [Event]
 addBlockMarker cap evts =
@@ -295,12 +295,19 @@ buildEventInfo spec' =
           <> TB.decimal parMaxCopied <> " bytes max par copied, "
           <> TB.decimal parTotCopied <> " bytes total par copied"
           <> maybe mempty (\val -> ", " <> TB.decimal val <> " bytes balanced par copied") parBalancedCopied
+        MemReturn{..} ->
+          "memory returned (mblocks): current(" <> TB.decimal current  <>
+                                   ") needed(" <> TB.decimal needed  <>
+                                   ") returned(" <> TB.decimal returned <> ")"
         HeapAllocated{..} ->
           "allocated on heap capset " <> TB.decimal heapCapset
           <> ": " <> TB.decimal allocBytes <> " total bytes till now"
         HeapSize{..} ->
           "size of heap capset " <> TB.decimal heapCapset
           <> ": " <> TB.decimal sizeBytes <> " bytes"
+        BlocksSize{..} ->
+          "blocks size of heap capset " <> TB.decimal heapCapset
+          <> ": " <> TB.decimal blocksSize <> " bytes"
         HeapLive{..} ->
           "live data in heap capset " <> TB.decimal heapCapset
           <> ": " <> TB.decimal liveBytes <> " bytes"
@@ -462,6 +469,11 @@ buildEventInfo spec' =
           <> " in " <> TB.fromText heapProfModule
           <> " at " <> TB.fromText heapProfSrcLoc
           <> if isCaf heapProfFlags then " CAF" else ""
+        InfoTableProv{..} ->
+         "Info Table: " <> TB.hexadecimal itInfo <> ":"
+                        <> TB.decimal itClosureDesc <> ":"
+                        <> TB.fromText itTableName
+                        <> " - " <> TB.fromText itSrcLoc
         HeapProfSampleBegin {..} ->
           "start heap prof sample " <> TB.decimal heapProfSampleEra
         HeapProfSampleEnd {..} ->
@@ -492,6 +504,39 @@ buildEventInfo spec' =
         UserBinaryMessage {..} ->
           "binary message " <> TB.fromText (replaceUnprintableWith '.' payload)
 
+        ConcMarkBegin    ->
+          "concurrent mark began"
+        ConcMarkEnd {..} ->
+          "concurrent mark ended: "
+          <> "marked " <> TB.decimal concMarkedObjectCount <> " objects"
+        ConcSyncBegin ->
+          "post-mark synchronization began"
+        ConcSyncEnd ->
+          "post-mark synchronization ended"
+        ConcSweepBegin ->
+          "concurrent sweep began"
+        ConcSweepEnd ->
+          "concurrent sweep ended"
+        ConcUpdRemSetFlush {..}  ->
+          "update remembered set flushed by " <> TB.decimal cap
+        NonmovingHeapCensus {..}  ->
+          "nonmoving heap census " <> TB.decimal (2^nonmovingCensusBlkSize :: Int)
+          <> ": " <> TB.decimal nonmovingCensusActiveSegs <> " active segments"
+          <> ", " <> TB.decimal nonmovingCensusFilledSegs <> " filled segments"
+          <> ", " <> TB.decimal nonmovingCensusLiveBlocks <> " live blocks"
+        TickyCounterDef {..}  ->
+          "ticky counter definition " <> TB.decimal tickyCtrDefId
+          <> ", " <>  "arity: " <> TB.decimal tickyCtrDefArity
+          <> ", " <> "def kinds: " <> TB.fromText tickyCtrDefKinds
+          <> ", " <> "name: " <> TB.fromText tickyCtrDefName
+        TickyCounterSample {..}  ->
+          "ticky counter sample " <> TB.decimal tickyCtrSampleId
+          <> ": " <> "entry count: " <> TB.decimal tickyCtrSampleEntryCount
+          <> ", " <> TB.decimal tickyCtrSampleAllocs <> " allocs"
+          <> ", " <> TB.decimal tickyCtrSampleAllocd <> " allocd"
+        TickyBeginSample ->
+          "ticky begin counter sample"
+
 -- | Replace unprintable bytes in the message with the replacement character
 replaceUnprintableWith
   :: Char -- ^ Replacement character
@@ -550,6 +595,7 @@ showHeapProfBreakdown breakdown = case breakdown of
   HeapProfBreakdownRetainer -> "retainer"
   HeapProfBreakdownBiography -> "biography"
   HeapProfBreakdownClosureType -> "closure type"
+  HeapProfBreakdownInfoTable -> "info table"
 
 ppEventLog :: EventLog -> String
 ppEventLog = TL.unpack . TB.toLazyText . buildEventLog
@@ -575,7 +621,10 @@ buildEventType (EventType num dsc msz) =
   <> maybe "variable" TB.decimal msz <> ")"
 
 -- | Pretty prints an 'Event', with clean handling for 'UnknownEvent'
-ppEvent :: IntMap EventType -> Event -> String
+ppEvent
+  :: IntMap EventType -- ^ Look up @'UnknownEvent'.'ref'@ to find a suitable description.
+  -> Event
+  -> String
 ppEvent imap = TL.unpack . TB.toLazyText . buildEvent imap
 
 buildEvent :: IntMap EventType -> Event -> TB.Builder
