@@ -1,5 +1,6 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE ViewPatterns #-}
 module GHC.RTS.Events.Binary
   ( -- * Readers
     getHeader
@@ -16,6 +17,7 @@ module GHC.RTS.Events.Binary
   , post782StopParser
   , parRTSParsers
   , binaryEventParsers
+  , tickyParsers
 
   -- * Writers
   , putEventLog
@@ -40,6 +42,7 @@ import Data.Binary.Put
 import qualified Data.Binary.Get as G
 import qualified Data.ByteString as B
 import qualified Data.Text as T
+import qualified Data.Text.Read as TR
 import qualified Data.Text.Encoding as TE
 import qualified Data.Vector.Unboxed as VU
 
@@ -175,6 +178,17 @@ standardParsers = [
                        , ..}
  )),
 
+ (FixedSizeParser EVENT_MEM_RETURN (sz_capset + 3*4) (do
+      heapCapset   <- get
+      current      <- get :: Get Word32
+      needed       <- get :: Get Word32
+      returned     <- get :: Get Word32
+      return $! MemReturn{ current = current
+                       , needed = needed
+                       , returned = returned
+                       , ..}
+ )),
+
  (FixedSizeParser EVENT_HEAP_ALLOCATED (sz_capset + 8) (do  -- (heap_capset, alloc_bytes)
       heapCapset <- get
       allocBytes <- get
@@ -186,6 +200,11 @@ standardParsers = [
       sizeBytes  <- get
       return HeapSize{..}
  )),
+ (FixedSizeParser EVENT_BLOCKS_SIZE (sz_capset + 8) (do  -- (heap_capset, blocks_size)
+      heapCapset <- get
+      blocksSize  <- get
+      return $! BlocksSize{..}
+ )),
 
  (FixedSizeParser EVENT_HEAP_LIVE (sz_capset + 8) (do  -- (heap_capset, live_bytes)
       heapCapset <- get
@@ -317,6 +336,27 @@ standardParsers = [
       str <- getText (num - sz_tid)
       return ThreadLabel{ thread      = tid
                         , threadlabel = str }
+    )),
+
+ (simpleEvent EVENT_CONC_MARK_BEGIN ConcMarkBegin),
+ (FixedSizeParser EVENT_CONC_MARK_END 4 (do -- (marked_object_count)
+      num <- get :: Get Word32
+      return ConcMarkEnd{ concMarkedObjectCount = num }
+    )),
+ (simpleEvent EVENT_CONC_SYNC_BEGIN ConcSyncBegin),
+ (simpleEvent EVENT_CONC_SYNC_END ConcSyncEnd),
+ (simpleEvent EVENT_CONC_SWEEP_BEGIN ConcSweepBegin),
+ (simpleEvent EVENT_CONC_SWEEP_END ConcSweepEnd),
+ (FixedSizeParser EVENT_CONC_UPD_REM_SET_FLUSH sz_cap (do -- (cap)
+      cap <- get :: Get CapNo
+      return ConcUpdRemSetFlush{ cap = fromIntegral cap }
+    )),
+ (FixedSizeParser EVENT_NONMOVING_HEAP_CENSUS 13 (do -- (blk_size, active_segs, filled_segs, live_blks)
+      nonmovingCensusBlkSize <- get :: Get Word8
+      nonmovingCensusActiveSegs <- get :: Get Word32
+      nonmovingCensusFilledSegs <- get :: Get Word32
+      nonmovingCensusLiveBlocks <- get :: Get Word32
+      return NonmovingHeapCensus{..}
     ))
  ]
 
@@ -790,6 +830,28 @@ heapProfParsers =
         ])
       (return ())
     return $! HeapProfCostCentre {..}
+  , VariableSizeParser EVENT_IPE $ do
+    payloadLen <- get :: Get Word16
+    itInfo <- get
+    itTableName <- getTextNul
+    itClosureDescText <- getTextNul
+    itClosureDesc <- either fail (return . fst) (TR.decimal itClosureDescText)
+    itTyDesc <- getTextNul
+    itLabel <- getTextNul
+    itModule <- getTextNul
+    itSrcLoc <- getTextNul
+    assert
+      (fromIntegral payloadLen == sum
+        [ 8 -- itInfo
+        , textByteLen itTableName
+        , textByteLen itClosureDescText
+        , textByteLen itTyDesc
+        , textByteLen itLabel
+        , textByteLen itModule
+        , textByteLen itSrcLoc
+        ])
+      (return ())
+    return $! InfoTableProv {..}
   , FixedSizeParser EVENT_HEAP_PROF_SAMPLE_BEGIN 8 $ do
     heapProfSampleEra <- get
     return $! HeapProfSampleBegin {..}
@@ -859,7 +921,33 @@ binaryEventParsers =
     return $! UserBinaryMessage { payload }
   ]
 
--- | String byte length in the eventlog format. It includes
+tickyParsers :: [EventParser EventInfo]
+tickyParsers =
+  [ VariableSizeParser EVENT_TICKY_COUNTER_DEF $ do
+    payloadLen         <- get :: Get Word16
+    tickyCtrDefId      <- get
+    tickyCtrDefArity   <- get
+    tickyCtrDefKinds   <- getTextNul
+    tickyCtrDefName    <- getTextNul
+    assert
+      (fromIntegral payloadLen == sum
+        [ 8 -- tickyCtrDefId
+        , 2 -- tickyCtrDefArity
+        , textByteLen tickyCtrDefKinds
+        , textByteLen tickyCtrDefName
+        ])
+      (return ())
+    return $! TickyCounterDef{..}
+  , FixedSizeParser EVENT_TICKY_COUNTER_SAMPLE (8*4) $ do
+    tickyCtrSampleId         <- get
+    tickyCtrSampleEntryCount <- get
+    tickyCtrSampleAllocs     <- get
+    tickyCtrSampleAllocd     <- get
+    return $! TickyCounterSample{..}
+  , simpleEvent EVENT_TICKY_BEGIN_SAMPLE TickyBeginSample
+  ]
+
+-- | String byte length in the event log format. It includes
 -- 1 byte for NUL.
 textByteLen :: T.Text -> Int
 textByteLen = (+1) . B.length . TE.encodeUtf8
@@ -891,12 +979,12 @@ putHeader (Header ets) = do
     putMarker EVENT_HET_END
     putMarker EVENT_HEADER_END
  where
-    putEventType (EventType n d msz) = do
+    putEventType (EventType n (TE.encodeUtf8 -> d) msz) = do
         putMarker EVENT_ET_BEGIN
         putType n
         putE $ fromMaybe 0xffff msz
-        putE (fromIntegral $ T.length d :: EventTypeDescLen)
-        putE d
+        putE (fromIntegral $ B.length d :: EventTypeDescLen)
+        putByteString d
         -- the event type header allows for extra data, which we don't use:
         putE (0 :: Word32)
         putMarker EVENT_ET_END
@@ -945,6 +1033,7 @@ eventTypeNum e = case e of
     GCStatsGHC{} -> EVENT_GC_STATS_GHC
     HeapAllocated{} -> EVENT_HEAP_ALLOCATED
     HeapSize{} -> EVENT_HEAP_SIZE
+    BlocksSize{} -> EVENT_BLOCKS_SIZE
     HeapLive{} -> EVENT_HEAP_LIVE
     HeapInfoGHC{} -> EVENT_HEAP_INFO_GHC
     CapCreate{} -> EVENT_CAP_CREATE
@@ -1002,6 +1091,19 @@ eventTypeNum e = case e of
     ProfSampleCostCentre {} -> EVENT_PROF_SAMPLE_COST_CENTRE
     ProfBegin {}            -> EVENT_PROF_BEGIN
     UserBinaryMessage {} -> EVENT_USER_BINARY_MSG
+    ConcMarkBegin {} -> EVENT_CONC_MARK_BEGIN
+    ConcMarkEnd {} -> EVENT_CONC_MARK_END
+    ConcSyncBegin {} -> EVENT_CONC_SYNC_BEGIN
+    ConcSyncEnd {} -> EVENT_CONC_SYNC_END
+    ConcSweepBegin {} -> EVENT_CONC_SWEEP_BEGIN
+    ConcSweepEnd {} -> EVENT_CONC_SWEEP_END
+    ConcUpdRemSetFlush {} -> EVENT_CONC_UPD_REM_SET_FLUSH
+    NonmovingHeapCensus {} -> EVENT_NONMOVING_HEAP_CENSUS
+    TickyCounterDef {} -> EVENT_TICKY_COUNTER_DEF
+    TickyCounterSample {} -> EVENT_TICKY_COUNTER_SAMPLE
+    InfoTableProv {} -> EVENT_IPE
+    MemReturn {} -> EVENT_MEM_RETURN
+    TickyBeginSample {} -> EVENT_TICKY_BEGIN_SAMPLE
 
 nEVENT_PERF_NAME, nEVENT_PERF_COUNTER, nEVENT_PERF_TRACEPOINT :: EventTypeNum
 nEVENT_PERF_NAME = EVENT_PERF_NAME
@@ -1031,7 +1133,7 @@ putEventSpec (RunThread t) =
 
 -- here we assume that ThreadStopStatus fromEnum matches the definitions in
 -- EventLogFormat.h
--- The standard encoding is used here, which is wrong for eventlogs
+-- The standard encoding is used here, which is wrong for event logs
 -- produced by GHC-7.8.2 ([Stop status in GHC-7.8.2] in EventTypes.hs
 putEventSpec (StopThread t s) = do
     putE t
@@ -1106,10 +1208,10 @@ putEventSpec (WakeupThread t c) = do
     putE t
     putCap c
 
-putEventSpec (ThreadLabel t l) = do
-    putE (fromIntegral (T.length l) + sz_tid :: Word16)
+putEventSpec (ThreadLabel t (TE.encodeUtf8 -> l)) = do
+    putE (fromIntegral (B.length l) + sz_tid :: Word16)
     putE t
-    putE l
+    putByteString l
 
 putEventSpec Shutdown =
     return ()
@@ -1164,6 +1266,12 @@ putEventSpec GCStatsGHC{..} = do
       Nothing -> return ()
       Just v  -> putE v
 
+putEventSpec MemReturn{..} = do
+    putE heapCapset
+    putE current
+    putE needed
+    putE returned
+
 putEventSpec HeapAllocated{..} = do
     putE heapCapset
     putE allocBytes
@@ -1172,6 +1280,10 @@ putEventSpec HeapSize{..} = do
     putE heapCapset
     putE sizeBytes
 
+putEventSpec BlocksSize{..} = do
+    putE heapCapset
+    putE blocksSize
+
 putEventSpec HeapLive{..} = do
     putE heapCapset
     putE liveBytes
@@ -1215,22 +1327,22 @@ putEventSpec (CapsetRemoveCap cs cp) = do
     putE cs
     putCap cp
 
-putEventSpec (RtsIdentifier cs rts) = do
-    putE (fromIntegral (T.length rts) + sz_capset :: Word16)
+putEventSpec (RtsIdentifier cs (TE.encodeUtf8 -> rts)) = do
+    putE (fromIntegral (B.length rts) + sz_capset :: Word16)
     putE cs
-    putE rts
+    putByteString rts
 
-putEventSpec (ProgramArgs cs as) = do
-    let sz_args = sum $ map ((+ 1) {- for \0 -} . T.length) as
+putEventSpec (ProgramArgs cs (map TE.encodeUtf8 -> as)) = do
+    let sz_args = sum (map ((+ 1) {- for \0 -} . B.length) as) - 1
     putE (fromIntegral sz_args + sz_capset :: Word16)
     putE cs
-    mapM_ putE (intersperse "\0" as)
+    mapM_ putByteString (intersperse "\0" as)
 
-putEventSpec (ProgramEnv cs es) = do
-    let sz_env = sum $ map ((+ 1) {- for \0 -} . T.length) es
+putEventSpec (ProgramEnv cs (map TE.encodeUtf8 -> es)) = do
+    let sz_env = sum (map ((+ 1) {- for \0 -} . B.length) es) - 1
     putE (fromIntegral sz_env + sz_capset :: Word16)
     putE cs
-    mapM_ putE $ intersperse "\0" es
+    mapM_ putByteString $ intersperse "\0" es
 
 putEventSpec (OsProcessPid cs pid) = do
     putE cs
@@ -1245,17 +1357,17 @@ putEventSpec (WallClockTime cs sec nsec) = do
     putE sec
     putE nsec
 
-putEventSpec (Message s) = do
-    putE (fromIntegral (T.length s) :: Word16)
-    putE s
+putEventSpec (Message (TE.encodeUtf8 -> s)) = do
+    putE (fromIntegral (B.length s) :: Word16)
+    putByteString s
 
-putEventSpec (UserMessage s) = do
-    putE (fromIntegral (T.length s) :: Word16)
-    putE s
+putEventSpec (UserMessage (TE.encodeUtf8 -> s)) = do
+    putE (fromIntegral (B.length s) :: Word16)
+    putByteString s
 
-putEventSpec (UserMarker s) = do
-    putE (fromIntegral (T.length s) :: Word16)
-    putE s
+putEventSpec (UserMarker (TE.encodeUtf8 -> s)) = do
+    putE (fromIntegral (B.length s) :: Word16)
+    putByteString s
 
 putEventSpec (UnknownEvent {}) = error "putEventSpec UnknownEvent"
 
@@ -1358,10 +1470,10 @@ putEventSpec (MerReleaseThread thread_id) =
 putEventSpec MerCapSleeping = return ()
 putEventSpec MerCallingMain = return ()
 
-putEventSpec PerfName{..} = do
-    putE (fromIntegral (T.length name) + sz_perf_num :: Word16)
+putEventSpec PerfName{name = (TE.encodeUtf8 -> name), ..} = do
+    putE (fromIntegral (B.length name) + sz_perf_num :: Word16)
     putE perfNum
-    putE name
+    putByteString name
 
 putEventSpec PerfCounter{..} = do
     putE perfNum
@@ -1427,3 +1539,39 @@ putEventSpec ProfBegin {..} = do
 putEventSpec UserBinaryMessage {..} = do
     putE (fromIntegral (B.length payload) :: Word16)
     putByteString payload
+
+putEventSpec ConcMarkBegin = return ()
+putEventSpec ConcMarkEnd {..} = do
+    putE concMarkedObjectCount
+putEventSpec ConcSyncBegin = return ()
+putEventSpec ConcSyncEnd = return ()
+putEventSpec ConcSweepBegin = return ()
+putEventSpec ConcSweepEnd = return ()
+putEventSpec ConcUpdRemSetFlush {..} = do
+    putCap cap
+putEventSpec NonmovingHeapCensus {..} = do
+    putE nonmovingCensusBlkSize
+    putE nonmovingCensusActiveSegs
+    putE nonmovingCensusFilledSegs
+    putE nonmovingCensusLiveBlocks
+putEventSpec TickyCounterDef {..} = do
+    putE tickyCtrDefId
+    putE tickyCtrDefArity
+    putE (T.unpack tickyCtrDefKinds)
+    putE (T.unpack tickyCtrDefName)
+putEventSpec TickyCounterSample {..} = do
+    putE tickyCtrSampleId
+    putE tickyCtrSampleEntryCount
+    putE tickyCtrSampleAllocs
+    putE tickyCtrSampleAllocd
+putEventSpec InfoTableProv{..} = do
+    putE itInfo
+    mapM_ (putE . T.unpack)
+      [ itTableName
+      , T.pack (show itClosureDesc)
+      , itTyDesc
+      , itLabel
+      , itModule
+      , itSrcLoc
+      ]
+putEventSpec TickyBeginSample = return ()
