@@ -5,6 +5,8 @@ files = map ("test/"++)
     [ "queens-ghc-6.12.1.eventlog"
     , "queens-ghc-7.0.2.eventlog"
     , "mandelbrot-mmc-2011-06-14.eventlog"
+    , "nonmoving-gc.eventlog"
+    , "nonmoving-gc-census.eventlog"
     , "parallelTest.eventlog"
     , "pre77stop.eventlog", "782stop.eventlog", "783stop.eventlog"
     , "sleep.h.eventlog"
@@ -18,6 +20,8 @@ files = map ("test/"++)
     , "time-prof.eventlog"
     , "trace-binary-event.eventlog"
     , "unicode.eventlog"
+    , "ticky-ticky.eventlog"
+    , "ticky-begin-sample.eventlog"
     ]
 
 
